/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "domstubs.idl"

[scriptable, uuid(9fd42950-25e7-11d4-8a7d-006008c844c3)]
interface nsIDOMPkcs11 : nsISupports
{
  long                      deletemodule(in DOMString moduleName);
  long                      addmodule(in DOMString moduleName,
                                      in DOMString libraryFullPath,
                                      in long cryptoMechanismFlags,
                                      in long cipherFlags);
};
