/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

interface nsIVariant;
interface nsIArray;

[scriptable, uuid(3f4cb2d0-5f7e-44a9-9f4f-370945f8db08)]
interface nsIDOMModalContentWindow : nsISupports
{
  /**
   * Readonly attribute containing an arbitrary JS value passed by the
   * code that opened the modal content window. A security check is
   * performed at access time, per spec.
   */
  readonly attribute nsIVariant            dialogArguments;

  /**
   * The return value that will be returned to the function that
   * opened the modal content window.
   */
  attribute nsIVariant                   returnValue;
};
